/**
  **************************************************************************
  * @file     usbh_hid_class.c
  * @brief    usb host hid class type
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */
 #include "usbh_ccid_class.h"
 #include "usb_conf.h"
 #include "usbh_core.h"
 #include "usbh_ctrl.h"
 #include <string.h>
 //#include "usbh_hid_mouse.h"
#include "usbh_hid_keyboard.h"

 /** @addtogroup AT32F423_middlewares_usbh_class
  * @{
  */

/** @defgroup USBH_hid_class
  * @brief usb host class hid demo
  * @{
  */

/** @defgroup USBH_hid_class_private_functions
  * @{
  */

 static usb_sts_type uhost_init_handler(void *uhost);
 static usb_sts_type uhost_reset_handler(void *uhost);
 static usb_sts_type uhost_request_handler(void *uhost);
 static usb_sts_type uhost_process_handler(void *uhost);
 static usb_sts_type uhost_desc_handler(void *uhost, usb_header_desc_type* pdesc);
 
 
 usbh_composite_type usbh_ccid;
 
 usbh_class_handler_type uhost_hid_class_handler =
 {
   uhost_init_handler,
   uhost_reset_handler,
   uhost_request_handler,
   uhost_process_handler,
   &usbh_ccid,
 };

 /**
  * @brief  usb host class desc handler
  * @param  uhost: to the structure of usbh_core_type
  *         pdata: to the descriptor data
  * @retval status: usb_sts_type status
  */
static usb_sts_type uhost_desc_handler(void *uhost, usb_header_desc_type* pdesc)
{
  uint8_t* pdata = (uint8_t*)pdesc;
  if(pdesc->bDescriptorType != 0x21)//Class Specific DEVICE Descriptor Type  
    return USB_NOT_SUPPORT;
  
  if(pdata[0]!=0x36)
    return USB_NOT_SUPPORT;

  if(pdata[2]!=0x10 || pdata[3]!=0x01) //bcdCCID=0x0110
    return USB_NOT_SUPPORT;
  
  #define GET_VAL16(p)  (((p)[0]<<0)|((p)[1]<<8))
  #define GET_VAL32(p)  (((p)[0]<<0)|((p)[1]<<8)|((p)[2]<<16)|((p)[3]<<24))
  usbh_ccid.ccid.class_desc.blength           = pdesc->bLength;
  usbh_ccid.ccid.class_desc.bType             = pdesc->bDescriptorType;
  usbh_ccid.ccid.class_desc.bcdCCID           = GET_VAL16(pdata+2);
  usbh_ccid.ccid.class_desc.bMaxSlotIndex     = pdata[4];
  usbh_ccid.ccid.class_desc.bVoltageSupport   = pdata[5];
  usbh_ccid.ccid.class_desc.dwProtocol        = GET_VAL32(pdata+6);
  usbh_ccid.ccid.class_desc.dwDefaultClock    = GET_VAL32(pdata+10);
  usbh_ccid.ccid.class_desc.dwMaxClock        = GET_VAL32(pdata+14);
  usbh_ccid.ccid.class_desc.bNumClockSupported  = pdata[18];
  usbh_ccid.ccid.class_desc.dwDataRate        = GET_VAL32(pdata+19);
  usbh_ccid.ccid.class_desc.dwMaxDataRate     = GET_VAL32(pdata+23);
  usbh_ccid.ccid.class_desc.bNumDataRateSupported = pdata[27];
  usbh_ccid.ccid.class_desc.dwMaxIFSD         = GET_VAL32(pdata+28);
  usbh_ccid.ccid.class_desc.dwSyncProtocol    = GET_VAL32(pdata+32);
  usbh_ccid.ccid.class_desc.dwMechanical      = GET_VAL32(pdata+36);
  usbh_ccid.ccid.class_desc.dwFeature         = GET_VAL32(pdata+40);
  usbh_ccid.ccid.class_desc.dwMaxCCIDMessageLength = GET_VAL32(pdata+44);
  usbh_ccid.ccid.class_desc.bClassGetResponse = pdata[48];
  usbh_ccid.ccid.class_desc.bClassEnvelope    = pdata[49];
  usbh_ccid.ccid.class_desc.wLcdLayout        = GET_VAL16(pdata+50);
  usbh_ccid.ccid.class_desc.bPinSupport       = pdata[52];
  usbh_ccid.ccid.class_desc.bMaxCCIDBusySlots = pdata[53];

  return USB_OK;
}

/**
  * @brief  usb host class desc handler
  * @param  uhost: to the structure of usbh_core_type
  *         pdata: to the descriptor data
  *         total_length: descriptor data length
  * @retval status: usb_sts_type status
  */
static usb_sts_type usb_ccid_parse_desc(void *uhost, uint8_t* pdata, uint32_t total_length)
{
  usb_header_desc_type* pdesc = (usb_header_desc_type*)pdata;
  uint16_t index_len = USB_DEVICE_CFG_DESC_LEN;
  int index_intf;

  while((index_intf < USBH_MAX_INTERFACE) && index_len < total_length)
  {
    pdesc = usbh_get_next_header((uint8_t *)pdesc, &index_len);
    if( USB_OK == uhost_desc_handler(uhost, pdesc) )
    {
      break;
    }            
    index_intf ++;
  }
  return USB_NOT_SUPPORT;
}  
 
  /**
  * @brief  usb host class init handler
  * @param  uhost: to the structure of usbh_core_type
  * @retval status: usb_sts_type status
  */
static usb_sts_type uhost_init_handler(void *uhost)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_OK;
  uint8_t hidx, eptidx = 0;
  usbh_ccid_type *pccid =  &usbh_ccid.ccid;
  usbh_hid_type *phid =  &usbh_ccid.hid;
  
  usbh_ccid.puhost = puhost;
  puhost->class_handler->pdata = &usbh_ccid;

  /* get ccid interface */
  hidx = usbh_find_interface(puhost, 0x0B, 0x00, 0x00);
  if(hidx == 0xFF)
  {
    USBH_DEBUG("Unsupport Device!");
    return USB_NOT_SUPPORT;
  }
  pccid->if_num = hidx;

  /* get ccid protocol */
  pccid->protocol = puhost->dev.cfg_desc.interface[hidx].interface.bInterfaceProtocol;


  for(eptidx = 0; eptidx < puhost->dev.cfg_desc.interface[hidx].interface.bNumEndpoints; eptidx ++)
  {
    usb_endpoint_desc_type *ept_desc = puhost->dev.cfg_desc.interface[hidx].endpoint+eptidx;
    uint8_t ept_chn = usbh_alloc_channel(puhost, ept_desc->bEndpointAddress);
    uint8_t idx = 0;
    usbh_hc_open(puhost, ept_chn, ept_desc->bEndpointAddress,
                      puhost->dev.address, ept_desc->bmAttributes&0xF,
                      ept_desc->wMaxPacketSize,
                      puhost->dev.speed);    
    
    if((EPT_BULK_TYPE == ept_desc->bmAttributes) & 0xF)
    { //BULK
      if(ept_desc->bEndpointAddress & 0x80)
      {
        idx = CCID_EP_BULK_IN;
      }
      else
      {
        idx = CCID_EP_BULK_OUT;
      }
    }
    else
    {
      idx = CCID_EP_INTR;
    }
    pccid->ep_info[idx].ept           = ept_desc->bEndpointAddress;
    pccid->ep_info[idx].maxpacket     = ept_desc->wMaxPacketSize;
    pccid->ep_info[idx].in_poll       = ept_desc->bInterval;
    pccid->ep_info[idx].chn           = ept_chn;
    usbh_set_toggle(puhost, ept_chn, 0);
  }
  
  pccid->ctrl_state = USB_CCID_STATE_IDLE;
  
  
  /* get hid interface */
  hidx = usbh_find_interface(puhost, USB_CLASS_CODE_HID, 0x01, 0xFF);
  if(hidx == 0xFF)
  {
    USBH_DEBUG("Unsupport Device!");
    return USB_NOT_SUPPORT;
  }
  phid->if_num = hidx;

  /* get hid protocol */
  phid->protocol = puhost->dev.cfg_desc.interface[hidx].interface.bInterfaceProtocol;


  for(eptidx = 0; eptidx < puhost->dev.cfg_desc.interface[hidx].interface.bNumEndpoints; eptidx ++)
  {
    usb_endpoint_desc_type *ept_desc = puhost->dev.cfg_desc.interface[hidx].endpoint+eptidx;
    uint8_t ept_chn = usbh_alloc_channel(puhost, ept_desc->bEndpointAddress);
    uint8_t idx = 0;
    usbh_hc_open(puhost, ept_chn, ept_desc->bEndpointAddress,
                      puhost->dev.address, ept_desc->bmAttributes&0xF,
                      ept_desc->wMaxPacketSize,
                      puhost->dev.speed);    
    
    if(ept_desc->bEndpointAddress & 0x80)
    {
      idx = 0;
    }
    else
    {
      idx = 1;
    }
    phid->ep_info[idx].ept           = ept_desc->bEndpointAddress;
    phid->ep_info[idx].maxpacket     = ept_desc->wMaxPacketSize;
    phid->ep_info[idx].in_poll       = ept_desc->bInterval;
    phid->ep_info[idx].chn           = ept_chn;
    usbh_set_toggle(puhost, ept_chn, 0);
  }
  
  phid->ctrl_state = USB_HID_STATE_IDLE;
  
  return status;
}

/**
  * @brief  usb host class reset handler
  * @param  uhost: to the structure of usbh_core_type
  * @retval status: usb_sts_type status
  */
static usb_sts_type uhost_reset_handler(void *uhost)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  
  usb_sts_type status = USB_OK;
  int i;
  if(puhost->class_handler->pdata == NULL)
  {
    return status;
  }
  //process ccid
  for(i=0; i<CCID_EP_MAX; ++i)
  {
    usbh_ccid_type *pccid = &usbh_ccid.ccid;
    if(pccid->ep_info[i].chn != 0)
    {
      usbh_free_channel(puhost, pccid->ep_info[i].chn);
      usbh_ch_disable(puhost, pccid->ep_info[i].chn);
      pccid->ep_info[i].chn = 0;
    }
  }
  //process hid
  for(i=0; i<2; ++i)
  {
    usbh_hid_type *phid = &usbh_ccid.hid;
    if(phid->ep_info[i].chn != 0)
    {
      usbh_free_channel(puhost, phid->ep_info[i].chn);
      usbh_ch_disable(puhost, phid->ep_info[i].chn);
      phid->ep_info[i].chn = 0;
    }
  }
  return status;
}

/**
  * @brief  usb host hid class get descriptor
  * @param  uhost: to the structure of usbh_core_type
  * @param  length: descriptor length
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_hid_get_desc(void *uhost, uint16_t length)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
//  usbh_ccid_type *pccid =  (usbh_ccid_type *)puhost->class_handler->pdata;
  usb_sts_type status = USB_WAIT;
  uint8_t bm_req;
  uint16_t wvalue;
  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    bm_req = USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_STANDARD;
    wvalue = (0x21 << 8) & 0xFF00;

    usbh_get_descriptor(puhost, length, bm_req,
                                 wvalue, puhost->rx_buffer);
  }
  else
  {
    if(usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE) == USB_OK)
    {
//      pccid->hid_desc.bLength = puhost->rx_buffer[0];
  //    pccid->hid_desc.bDescriptorType = puhost->rx_buffer[1];
    //  pccid->hid_desc.bcdHID = SWAPBYTE(puhost->rx_buffer+2);
//      pccid->hid_desc.bCountryCode = puhost->rx_buffer[4];
      //pccid->hid_desc.bNumDescriptors = puhost->rx_buffer[5];
      //pccid->hid_desc.bReportDescriptorType = puhost->rx_buffer[6];
      //pccid->hid_desc.wItemLength = SWAPBYTE(puhost->rx_buffer+7);
      status = USB_OK;
    }
  }
  return status;
}

/**
  * @brief  usb host hid class get report
  * @param  uhost: to the structure of usbh_core_type
  * @param  length: reprot length
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_ccid_get_clock_freq(void *uhost, uint16_t length)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;

  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_D2H | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_STANDARD;
    puhost->ctrl.setup.bRequest      = 0x2;
    puhost->ctrl.setup.wValue        = 0;
    puhost->ctrl.setup.wIndex        = usbh_ccid.ccid.if_num;
    puhost->ctrl.setup.wLength       = length;

    usbh_ctrl_request(uhost, puhost->rx_buffer, length);
  }
  else
  {
    if(usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE) == USB_OK)
    {
      status = USB_OK;
    }
  }
  return status;
}

/**
  * @brief  usb host hid class get report
  * @param  uhost: to the structure of usbh_core_type
  * @param  length: reprot length
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_ccid_get_data_rate(void *uhost, uint16_t length)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;

  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_D2H | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_STANDARD;
    puhost->ctrl.setup.bRequest      = 0x3;
    puhost->ctrl.setup.wValue        = 0;
    puhost->ctrl.setup.wIndex        = usbh_ccid.ccid.if_num;
    puhost->ctrl.setup.wLength       = length;

    usbh_ctrl_request(uhost, puhost->rx_buffer, length);
  }
  else
  {
    if(usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE) == USB_OK)
    {
      status = USB_OK;
    }
  }
  return status;
}


/**
  * @brief  usb host hid class set idle
  * @param  uhost: to the structure of usbh_core_type
  * @param  id: id
  * @param  dr: dr
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_ccid_abort(void *uhost, uint8_t seq, uint8_t slot)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;
  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_H2D | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_CLASS;
    puhost->ctrl.setup.bRequest     = 0x01;
    puhost->ctrl.setup.wValue       = (seq << 8) | slot;
    puhost->ctrl.setup.wIndex       = usbh_ccid.ccid.if_num;
    puhost->ctrl.setup.wLength      = 0;
    usbh_ctrl_request(puhost, 0, 0);
  }
  else
  {
    status = usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE);
    if(status == USB_OK || status == USB_NOT_SUPPORT)
    {
      status = USB_OK;
    }
  }
  return status;
}
/**
  * @brief  usb host hid class get report
  * @param  uhost: to the structure of usbh_core_type
  * @param  length: reprot length
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_hid_get_report(void *uhost, uint16_t length)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;

  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_D2H | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_STANDARD;
    puhost->ctrl.setup.bRequest      = USB_STD_REQ_GET_DESCRIPTOR;
    puhost->ctrl.setup.wValue        = (0x22 << 8) & 0xFF00;;
    puhost->ctrl.setup.wIndex        = usbh_ccid.hid.if_num;
    puhost->ctrl.setup.wLength       = length;

    usbh_ctrl_request(uhost, puhost->rx_buffer, length);
  }
  else
  {
    if(usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE) == USB_OK)
    {
      status = USB_OK;
    }
  }
  return status;
}

/**
  * @brief  usb host hid class set idle
  * @param  uhost: to the structure of usbh_core_type
  * @param  id: id
  * @param  dr: dr
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_hid_set_idle(void *uhost, uint8_t id, uint8_t dr)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;
  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_H2D | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_CLASS;
    puhost->ctrl.setup.bRequest     = USB_HID_SET_IDLE;
    puhost->ctrl.setup.wValue       = (dr << 8) | id;
    puhost->ctrl.setup.wIndex       = usbh_ccid.hid.if_num;
    puhost->ctrl.setup.wLength      = 0;
    usbh_ctrl_request(puhost, 0, 0);
  }
  else
  {
    status = usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE);
    if(status == USB_OK || status == USB_NOT_SUPPORT)
    {
      status = USB_OK;
    }
  }
  return status;
}

/**
  * @brief  usb host hid class set protocol
  * @param  uhost: to the structure of usbh_core_type
  * @param  protocol: portocol number
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_hid_set_protocol(void *uhost, uint8_t protocol)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usb_sts_type status = USB_WAIT;
  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_H2D | USB_REQ_RECIPIENT_INTERFACE | USB_REQ_TYPE_CLASS;
    puhost->ctrl.setup.bRequest = USB_HID_SET_PROTOCOL;
    puhost->ctrl.setup.wValue   = protocol;
    puhost->ctrl.setup.wIndex   = usbh_ccid.hid.if_num;;
    puhost->ctrl.setup.wLength  = 0;
    usbh_ctrl_request(puhost, 0, 0);
  }
  else
  {
    status = usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE);
    if(status == USB_OK || status == USB_NOT_SUPPORT)
    {
      status = USB_OK;
    }
  }
  return status;
}

/**
  * @brief  usb host clear feature
  * @param  uhost: to the structure of usbh_core_type
  * @param  ept_num: endpoint number
  * @param  hc_num: channel number
  * @retval status: usb_sts_type status
  */
usb_sts_type usbh_clear_endpoint_feature(usbh_core_type *uhost, uint8_t ept_num, uint8_t hc_num)
{
  usb_sts_type status = USB_WAIT;
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  uint8_t bm_req;
  bm_req = USB_REQ_RECIPIENT_ENDPOINT | USB_REQ_TYPE_STANDARD;
  if(puhost->ctrl.state == CONTROL_IDLE)
  {
    puhost->ctrl.setup.bmRequestType = USB_DIR_H2D | bm_req;
    puhost->ctrl.setup.bRequest = USB_STD_REQ_CLEAR_FEATURE;
    puhost->ctrl.setup.wValue = 0x00;
    puhost->ctrl.setup.wLength = 0;
    puhost->ctrl.setup.wIndex = ept_num;
    if((ept_num & 0x80) == USB_DIR_D2H)
    {
      puhost->hch[hc_num].toggle_in = 0;
    }
    else
    {
      puhost->hch[hc_num].toggle_out = 0;
    }
    status = usbh_ctrl_request(puhost, 0, 0);
  }
  if(usbh_ctrl_result_check(puhost, CONTROL_IDLE, ENUM_IDLE) == USB_OK)
  {
    status = USB_OK;
  }
  return status;
}

/**
  * @brief  usb host hid class request handler
  * @param  uhost: to the structure of usbh_core_type
  * @retval status: usb_sts_type status
  */
static usb_sts_type uhost_request_handler(void *uhost)
{
  usb_sts_type status = USB_WAIT;
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usbh_ccid_type *pccid = &usbh_ccid.ccid;
  usbh_hid_type  *phid  = &usbh_ccid.hid;

  switch(phid->ctrl_state)
  {
    case USB_HID_STATE_GET_CCID_DESC:
      if(puhost->ctrl.state == CONTROL_IDLE)
      {
        usbh_get_configure_descriptor(uhost, puhost->dev.cfg_desc.cfg.wTotalLength);
      }

      if(usbh_ctrl_result_check(uhost, CONTROL_IDLE, ENUM_GET_MFC_STRING) == USB_OK)
      {
        usb_ccid_parse_desc(uhost, puhost->rx_buffer, puhost->dev.cfg_desc.cfg.wTotalLength);
        phid->ctrl_state = USB_HID_STATE_GET_REPORT;
      }
      break;
    case USB_HID_STATE_IDLE:
      phid->ctrl_state = USB_HID_STATE_GET_CCID_DESC;
      break;
    case USB_HID_STATE_GET_REPORT:
      if(usbh_hid_get_report(puhost, 142) == USB_OK)
      {
        phid->ctrl_state = USB_HID_STATE_SET_IDLE;
      }
      break;
    case USB_HID_STATE_SET_IDLE:
      if(usbh_hid_set_idle(puhost, 0, 0) == USB_OK)
      {
        phid->ctrl_state = USB_HID_STATE_SET_PROTOCOL;
      }
      break;
    case USB_HID_STATE_SET_PROTOCOL:
      if(usbh_hid_set_protocol(puhost, 0) == USB_OK)
      {
        phid->ctrl_state = USB_HID_STATE_COMPLETE;
      }
      break;
    case USB_HID_STATE_COMPLETE:
      pccid->state = phid->state = USB_HID_INIT;
      status = USB_OK;
      break;
    default:
      break;
  }

  return status;
}

/**
  * @brief  start to ccid class request
  * @param  req: class request
  * @retval none
  */
void ccid_start_class_request(usb_ccid_ctrl_state_type req)
{
  if(  req == USB_CCID_STATE_ABORT 
    || req == USB_CCID_STATE_GET_CLOCK_FREQ
    || req == USB_CCID_STATE_GET_DATA_RATE ){
    usbh_ccid.ccid.ctrl_state = req;
  }
}


/**
  * @brief  user function when message transfer complete
  * @param  none
  * @retval none
  */
__weak void user_message_transfer_complete()
{
  //do your job there
}

/**
  * @brief  user function when receive device respond complete
  * @param  none
  * @retval none
  */
__weak void user_message_respond_complete()
{
  //do your job there
}

/**
  * @brief  usb host ccis process bulk out
  * @param  puhost: to the structure of usbh_core_type
  * @retval none
  */
static void ccid_process_out(usbh_core_type *puhost)
{
  usbh_ccid_type *pccid =  &usbh_ccid.ccid; 
  urb_sts_type urb_status;
  usb_sts_type clr_status;
  uint16_t len;
  
  switch(pccid->out.state)
  {
    case DATA_STATE_IDLE:
      break;
    case DATA_STATE_XFER:
      {
        if(pccid->out.len_remain > pccid->ep_info[CCID_EP_BULK_OUT].maxpacket)
        {
          len = pccid->ep_info[CCID_EP_BULK_OUT].maxpacket;
        }
        else
        {
          len = pccid->out.len_remain;
        }
        if(len)
        {
          pccid->out.state = DATA_STATE_XFER_WAIT;
          pccid->out.len_xfering = len;
          usbh_bulk_send(puhost, pccid->ep_info[CCID_EP_BULK_OUT].chn, pccid->out.data, len);
        }
        else
        {
          pccid->out.state = DATA_STATE_COMPLETE;
        }
      }
      break;
    case DATA_STATE_XFER_WAIT:
      urb_status = usbh_get_urb_status(puhost, pccid->ep_info[CCID_EP_BULK_OUT].chn);
      if(urb_status == URB_DONE)
      {
        pccid->out.data       += pccid->out.len_xfering;
        pccid->out.len_remain -= pccid->out.len_xfering;
        pccid->out.state       = DATA_STATE_XFER;
      }
      else if(urb_status == URB_STALL)
      {
        pccid->out.state = DATA_STATE_XFER_ERROR;
      }
      break;
    case DATA_STATE_XFER_ERROR:
      clr_status = usbh_clear_ept_feature(puhost, pccid->ep_info[CCID_EP_BULK_OUT].ept, pccid->ep_info[CCID_EP_BULK_OUT].chn);
      if(clr_status == USB_OK)
      {
        usbh_set_toggle(puhost, pccid->ep_info[CCID_EP_BULK_OUT].chn, 0);
      }
      break;
    case DATA_STATE_COMPLETE:
      pccid->out.state = DATA_STATE_IDLE;
      user_message_transfer_complete();
      break;
    default:
      break;
  }
}

/**
  * @brief  usb host ccis process bulk in
  * @param  puhost: to the structure of usbh_core_type
  * @retval none
  */
static void ccid_process_in(usbh_core_type *puhost)
{
  urb_sts_type urb_status;
  usb_sts_type clr_status;
  usbh_ccid_type *pccid =  &usbh_ccid.ccid; 
  uint16_t len;
  
  switch(pccid->in.state)
  {
    case DATA_STATE_IDLE:
      break;
    case DATA_STATE_XFER:
      {
        if(pccid->in.len_remain > pccid->ep_info[CCID_EP_BULK_IN].maxpacket)
        {
          len = pccid->ep_info[CCID_EP_BULK_IN].maxpacket;
        }
        else
        {
          len = pccid->in.len_remain;
        }
        if(len)
        {
          pccid->in.state = DATA_STATE_XFER_WAIT;
          pccid->in.len_xfering = len;
          usbh_bulk_recv(puhost, pccid->ep_info[CCID_EP_BULK_IN].chn, pccid->in.data, len);
        }
        else
        {
          pccid->in.state = DATA_STATE_COMPLETE;
        }
      }
      break;
    case DATA_STATE_XFER_WAIT:
      urb_status = usbh_get_urb_status(puhost, pccid->ep_info[CCID_EP_BULK_OUT].chn);
      if(urb_status == URB_DONE)
      {
        pccid->in.data       += pccid->in.len_xfering;
        pccid->in.len_remain -= pccid->in.len_xfering;
        pccid->in.state       = DATA_STATE_XFER;
      }
      else if(urb_status == URB_STALL)
      {
        pccid->in.state = DATA_STATE_XFER_ERROR;
      }
      break;
    case DATA_STATE_XFER_ERROR:
      clr_status = usbh_clear_ept_feature(puhost, pccid->ep_info[CCID_EP_BULK_IN].ept, pccid->ep_info[CCID_EP_BULK_IN].chn);
      if(clr_status == USB_OK)
      {
        usbh_set_toggle(puhost, pccid->ep_info[CCID_EP_BULK_IN].chn, 0);
      }
      break;
    case DATA_STATE_COMPLETE:
      pccid->in.state = DATA_STATE_IDLE;
      user_message_respond_complete();
      break;
    default:
      break;
    
  }
}

/**
  * @brief  user main process routine, non-blocking
  * @param  none: user can modify if needed.
  * @retval none: user can modify if needed.
  */
void user_process_handler(void)
{
  //user code begin
  
  //user code end
}

/**
  * @brief  user routine to decode ccid interrupt-in message
  * @param  data: interrupt-in message.
  * @retval none.
  */
void usbh_ccid_interrupt_decode(uint8_t *data)
{
  //user code begin
  
  //user code end
}

/**
  * @brief  usb host hid class process handler
  * @param  uhost: to the structure of usbh_core_type
  * @retval status: usb_sts_type status
  */
static usb_sts_type uhost_process_handler(void *uhost)
{
  usbh_core_type *puhost = (usbh_core_type *)uhost;
  usbh_hid_type *phid =  &usbh_ccid.hid;
  usbh_ccid_type *pccid =  &usbh_ccid.ccid;
  urb_sts_type urb_status;

  //user process
  user_process_handler();
  
  //process bulk
  ccid_process_out(puhost);
  ccid_process_in(puhost);
  //process ccid interrupt
  switch(pccid->ctrl_state)
  {
    case USB_CCID_STATE_GET_CLOCK_FREQ:
      if(pccid->class_desc.bNumClockSupported){
        if(usbh_ccid_get_clock_freq(puhost, pccid->class_desc.bNumClockSupported*4) == USB_OK)
        {
          pccid->ctrl_state = USB_CCID_STATE_IDLE;
        }
      }
      break;
    case USB_CCID_STATE_ABORT:
      if(usbh_ccid_abort(puhost, 0, 0) == USB_OK) 
      {
        pccid->ctrl_state = USB_CCID_STATE_IDLE;
      }
      break;
    case USB_CCID_STATE_GET_DATA_RATE:
      if(pccid->class_desc.bNumDataRateSupported){
        if(usbh_ccid_get_data_rate(puhost, pccid->class_desc.bNumDataRateSupported*4) == USB_OK)
        {
          pccid->ctrl_state = USB_CCID_STATE_IDLE;
        }
      }
      break;
    default:
      break;
  }
  //process hid interrupt
  switch(pccid->state)
  {
    
    case USB_HID_INIT:
      pccid->state = USB_HID_GET;
      break;

    case USB_HID_GET:
      usbh_interrupt_recv(puhost, pccid->ep_info[CCID_EP_INTR].chn, (uint8_t *)pccid->buffer, pccid->ep_info[CCID_EP_INTR].maxpacket);
      pccid->state = USB_HID_POLL;
      pccid->poll_timer = usbh_get_frame(puhost->usb_reg);
      break;

    case USB_HID_POLL:
      if((usbh_get_frame(puhost->usb_reg) - pccid->poll_timer) >= pccid->ep_info[CCID_EP_INTR].in_poll )
      {
        pccid->state = USB_HID_GET;
      }
      else
      {
        urb_status = usbh_get_urb_status(puhost, pccid->ep_info[CCID_EP_INTR].chn);
        if(urb_status == URB_DONE)
        {
          puhost->urb_state[pccid->ep_info[CCID_EP_INTR].chn] = URB_IDLE;
          
          usbh_ccid_interrupt_decode((uint8_t *)pccid->buffer);
        }
        else if(urb_status == URB_STALL)
        {
          if(usbh_clear_endpoint_feature(puhost, pccid->ep_info[CCID_EP_INTR].ept, pccid->ep_info[CCID_EP_INTR].chn) ==  USB_OK)
          {
            pccid->state = USB_HID_GET;
          }
        }
      }
      break;

    default:
      break;
  }
  switch(phid->state)
  {
    case USB_HID_INIT:
      phid->state = USB_HID_GET;
      break;

    case USB_HID_GET:
      usbh_interrupt_recv(puhost, phid->ep_info[0].chn, (uint8_t *)phid->buffer, phid->ep_info[0].maxpacket);
      phid->state = USB_HID_POLL;
      phid->poll_timer = usbh_get_frame(puhost->usb_reg);
      break;

    case USB_HID_POLL:
      if((usbh_get_frame(puhost->usb_reg) - phid->poll_timer) >= phid->ep_info[0].in_poll )
      {
        phid->state = USB_HID_GET;
      }
      else
      {
        urb_status = usbh_get_urb_status(puhost, phid->ep_info[0].chn);
        if(urb_status == URB_DONE)
        {
          puhost->urb_state[phid->ep_info[0].chn] = URB_IDLE;
          
          usbh_hid_keyboard_decode((uint8_t *)phid->buffer);
        }
        else if(urb_status == URB_STALL)
        {
          if(usbh_clear_endpoint_feature(puhost, phid->ep_info[0].ept, phid->ep_info[0].chn) ==  USB_OK)
          {
            phid->state = USB_HID_GET;
          }
        }
      }
      break;

    default:
      break;
  }
  return USB_OK;
}


/**
  * @brief  usb host ccid class start transmission handler
  * @param  uhost: to the structure of usbh_core_type
  * @param  data: tx data pointer
  * @param  len: tx data len
  * @retval status: usb_sts_type status
  */
void ccid_start_transmission(usbh_core_type *uhost, uint8_t *data, uint32_t len)
{
  usbh_ccid_type *pccid =  &usbh_ccid.ccid;
  if(pccid->out.state == DATA_STATE_IDLE)
  {
    pccid->out.data       = data;
    pccid->out.len_remain = len;
    pccid->out.state      = DATA_STATE_XFER;
  }
}

/**
  * @brief  usb host ccid class start reception handler
  * @param  uhost: to the structure of usbh_core_type
  * @param  data: receive data pointer
  * @param  len: receive data len
  * @retval status: usb_sts_type status
  */
void ccid_start_reception(usbh_core_type *uhost, uint8_t *data, uint32_t len)
{
  usbh_ccid_type *pccid =  &usbh_ccid.ccid;
  if(pccid->in.state == DATA_STATE_IDLE)
  {
    pccid->in.data        = data;
    pccid->in.len_remain  = len;
    pccid->in.state       = DATA_STATE_XFER;
  }
}


const uint8_t get_slot_cmd[]={0x65, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00 };
uint8_t reponse_buf[64];
uint8_t cmd_xfr_buf[64];
void ccid_test_PC_to_RDP_GetSlotStatus(void)
{
  ccid_start_transmission(usbh_ccid.puhost, (uint8_t*)get_slot_cmd, 10 );
  ccid_start_reception   (usbh_ccid.puhost, (uint8_t*)reponse_buf, 10 );
}

void ccid_test_PC_to_RDP_XfrBlock(void)
{
  #define TEST_BLOCK_LEN  (usbh_ccid.ccid.class_desc.dwMaxCCIDMessageLength-10)
  cmd_xfr_buf[0]=0x6F;
  cmd_xfr_buf[1]=(TEST_BLOCK_LEN >> 0)& 0xFF;
  cmd_xfr_buf[2]=(TEST_BLOCK_LEN >> 8)& 0xFF;
  cmd_xfr_buf[3]=(TEST_BLOCK_LEN >>16)& 0xFF;
  cmd_xfr_buf[4]=(TEST_BLOCK_LEN >>24)& 0xFF;
  cmd_xfr_buf[5]=0;
  cmd_xfr_buf[6]=1;
  cmd_xfr_buf[7]=0xFF;
  ccid_start_transmission(usbh_ccid.puhost, (uint8_t*)cmd_xfr_buf, TEST_BLOCK_LEN+10 );
  ccid_start_reception   (usbh_ccid.puhost, (uint8_t*)reponse_buf, 10 );
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */
